"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const grayMatter = require("gray-matter");
const util_1 = require("util");
const hasha = require("hasha");
const stableStringify = require("json-stable-stringify");
const remark = require("remark");
const slug = require('remark-slug');
const hljs = require('remark-highlight.js');
const html = require('remark-html');
const emoji = require('remark-gemoji-to-emoji');
const autolinkHeadings = require('remark-autolink-headings');
const inlineLinks = require('remark-inline-links');
const toc = require('remark-toc');
async function markdowner(markdownString, opts) {
    var _a;
    const hash = makeHash(markdownString, opts);
    const defaults = {
        frontmatter: false,
        runBefore: [],
        toc: false,
    };
    opts = Object.assign(defaults, opts);
    let data = {};
    let content = markdownString;
    // Check the cache for preprocessed markdown
    if (opts.cache) {
        let existing = false;
        try {
            existing = await opts.cache.get(hash);
        }
        catch (err) {
            if (!err.notFound)
                console.error(err);
        }
        if (existing)
            return existing;
    }
    if (opts.frontmatter) {
        const parsed = grayMatter(markdownString);
        data = parsed.data;
        content = parsed.content;
    }
    if (opts.toc) {
        opts.runBefore = [toc];
    }
    const renderer = remark()
        .use((_a = opts.runBefore) !== null && _a !== void 0 ? _a : [])
        .use(slug)
        .use(autolinkHeadings, { behavior: 'wrap' })
        .use(inlineLinks)
        .use(emoji)
        .use(hljs)
        .use(html, { sanitize: false });
    const md = await util_1.promisify(renderer.process)(content);
    Object.assign(data, { content: md.contents });
    // Save processed markdown in cache
    if (opts.cache)
        await opts.cache.put(hash, data);
    return data;
}
exports.markdowner = markdowner;
// Create a unique hash from the given input (markdown + options object)
function makeHash(markdownString, opts) {
    // Copy existing opts object to avoid mutation
    const hashableOpts = Object.assign({}, opts);
    // Ignore `cache` prop when creating hash
    delete hashableOpts.cache;
    // Deterministic stringifier gets a consistent hash from stringified results
    // object keys are sorted to ensure {a:1, b:2} has the same hash as {b:2, a:1}
    // empty object should become an empty string, not {}
    const optsString = Object.keys(hashableOpts).length
        ? stableStringify(hashableOpts)
        : '';
    return hasha(markdownString + optsString);
}
